Imports System.Drawing
Imports System.Windows.Forms

' Reversibles Zeichnen (DrawReversibleFrame(), DrawReversibleLine(), 
' FillReversibleRectangle() zeichnet Umkehrwerte der bestehenden 
' Bildschirm-Pixel. Infolgedessen ist zum Lschen einfach exakt dieselbe 
' Zeichnung zu wiederholen
'
' Vorteil ist auch: Durch FarbInversion gezeichnete Linien sind auf 
'  jedem Untergrund sichtbar
' Nachteil: In einigen Szenarien kann ein normaler Zeichenvorgang den 
'  reversiblen bermalen. Will sich nun der Reversible durch Neuzeichnung 
'  lschen, so wird er erst erneut sichtbar, und verbleibt als unerwnschter 
'  Zeichnungsrest.

Public Class RubberRectangle

    Private _Rectangle As Rectangle
    Private _anchor As Size

    ''' <summary> das Auswahlrechteck in BildschirmKoordinaten </summary>
    Public ReadOnly Property Rectangle() As Rectangle
        Get
            Dim rct As Rectangle = _Rectangle
            ' DrawReversibleFrame verarbeitet zwar auch Rechtecke mit negativer 
            '  Breite oder Hhe, aber doch lieber umrechnen. 
            If rct.Width < 0 Then
                rct.X += rct.Width
                rct.Width = -rct.Width
            End If
            If rct.Height < 0 Then
                rct.Y += rct.Height
                rct.Height = -rct.Height
            End If
            Return rct
        End Get
    End Property

    Public Sub Start(ByVal sender As Object, ByVal e As MouseEventArgs)
        If e.Button <> MouseButtons.Left Then _
            Throw New ArgumentException( _
                "Rubberband darf nur bei gedrckter linker Maustaste " & _
                "gestartet werden", "e.Button")

        _Rectangle = New Rectangle(Control.MousePosition, Size.Empty)
        _anchor = New Size(Rectangle.X, Rectangle.Y)

        ' Erstmaliges Zeichnen
        DrawRect()

        Dim ctl As Control = DirectCast(sender, Control)
        AddHandler ctl.MouseMove, AddressOf ctl_MouseMove
        AddHandler ctl.MouseUp, AddressOf ctl_MouseUp
    End Sub

    Private Sub ctl_MouseMove(ByVal sender As Object, ByVal e As MouseEventArgs)
        ' Lschen
        DrawRect()

        Dim pt As Point = Control.MousePosition
        _Rectangle.Size = New Size(pt - _anchor)

        ' Neuzeichnen
        DrawRect()
    End Sub

    Private Sub ctl_MouseUp(ByVal sender As Object, ByVal e As MouseEventArgs)
        ' Letztmaliges Lschen
        DrawRect()

        Dim ctl As Control = DirectCast(sender, Control)
        RemoveHandler ctl.MouseMove, AddressOf ctl_MouseMove
        RemoveHandler ctl.MouseUp, AddressOf ctl_MouseUp
    End Sub

    Private Sub DrawRect()
        ControlPaint.DrawReversibleFrame(_Rectangle, _
            Color.Yellow, FrameStyle.Thick)
    End Sub

End Class
